import os 
import pandas as pd 
import shutil


def csv_slice(csv_path, target_file, start_end_index=(0,1)):
    """
    大文件截取一个切片，便于开发测试使用
    """
    fil = pd.read_csv(csv_path)
    fil = fil.iloc[start_end_index[0]:start_end_index[1]]
    fil.to_csv(target_file,index=False)

def csv_slice_small(csv_path, target_dir, max_row_one_csv=100000):
    """
    将一个大的CSV文件拆分成一个个小的CSV文件
    """
    fil = pd.read_csv(csv_path)
    # print(fil.shape[0])
    (filepath, tempfilename) = os.path.split(csv_path)
    (filesname, extension) = os.path.splitext(tempfilename)
        
    
    all_row_counts = fil.shape[0]

    single_file_rows = max_row_one_csv 
    if all_row_counts > single_file_rows*3:  # 是批次的3倍才值得拆一下，1个多批次的数据合在一起计算就可以了
        start_index = 0
        while start_index < all_row_counts:
            end_index = start_index + single_file_rows

            if end_index > all_row_counts:
                end_index = all_row_counts

            one_batch_data = fil.iloc[start_index:end_index]
            one_csv_path = os.path.join(target_dir,"{}_{}_{}".format(filesname,start_index, end_index)+".csv")
            one_batch_data.to_csv(one_csv_path, index=False)
            start_index = end_index 
    else:
        start_index = 0
        end_index = all_row_counts
        one_csv_path = os.path.join(target_dir,"{}_{}_{}".format(filesname,start_index, end_index)+".csv")
        one_batch_data.to_csv(one_csv_path, index=False)
    


def mkdir(path):
	folder = os.path.exists(path)
	if not folder:                   #判断是否存在文件夹如果不存在则创建为文件夹
		os.makedirs(path)            #makedirs 创建文件时如果路径不存在会创建这个路径

def mkfil(fil):
    """
    创建文件
    """
    _file = open(fil,'w')
    _file.close()

def parentdir(fil):
    father_path=os.path.abspath(os.path.dirname(fil)+os.path.sep+".")
    return father_path


 
def load_file():
    # 获取当前文件路径
    current_path = os.path.abspath(__file__)
    # 获取当前文件的父目录
    father_path = os.path.abspath(os.path.dirname(current_path) + os.path.sep + ".")
    # config.ini文件路径,获取当前目录的父目录的父目录与congig.ini拼接
    config_file_path=os.path.join(os.path.abspath(os.path.dirname(current_path) + os.path.sep + ".."),'config.ini')
    print('当前目录:' + current_path)
    print('当前父目录:' + father_path)
    print('config.ini路径:' + config_file_path)

def filname_nosuffix(file_name):
    if os.path.exists(file_name):
       name_path = os.path.splitext(file_name)[0]  # 将文件名和扩展名(后缀)分开
       _name1 = os.path.split(name_path)[-1]
    return _name1  

def fil_suffix(file_name):
    """
    获取文件后缀
    """
    sfx = os.path.splitext(file_name)[1]  # 将文件名和扩展名(后缀)分开
    return sfx 

def rmfil(fil):
    if os.path.exists(r'{}'.format(fil)):
        shutil.rmtree(r'{}'.format(fil))


def write(text,file_name):              #定义函数名
    """
    写入少量内容到文件
    """
    # b = os.getcwd()[:-4] + 'new\\'
 
    if os.path.exists(file_name):     #判断当前路径是否存在，没有则创建new文件夹
        # os.makedirs(b)
 
        file = open(file_name,'w')
    
        file.write(text)        #写入内容信息
    
        file.close()

import time

def log(smg, log_file="main.log"):
    # BASE_DIR=os.path.dirname(os.path.abspath(__file__))
    # log_file = "./main.log"
    # log_file = os.path.join(BASE_DIR,log_file)
    # print(log_file)
    with open(log_file,"a+",encoding="utf-8") as f:
        tim = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
        f.write("[{}] {} \n".format(tim,smg))


def log_with_max(msg, fil,max_file_size=10*1024*1024):

    if fil:
        log_file = fil
    else:
        log_file = "/tmp/train.log"
    fil_size = os.path.getsize(log_file)
    if fil_size > max_file_size:
        # 写入空文件
        with open(log_file,"w",encoding="utf-8") as f:
            tim = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
            f.write("[{}] {} \n".format(tim,msg))
    else:
        with open(log_file,"a+",encoding="utf-8") as f:
            tim = time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
            f.write("[{}] {} \n".format(tim,msg))


def iswin():
    try:
        import platform
        if platform.system().lower() == 'windows':
            return True
        elif platform.system().lower() == 'linux':
            return False
    except:
        return True


if __name__=="__main__":
    fil = "/opt/aisty/73_code/build/lib/aisty"
    rmfil(fil)
